/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.generator;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

/**
 * An abstract implementation of a generator.
 * This class allows the generation of any document types (Java, XML, ...).
 * It manage the output file and the indentation into the generated file.
 *
 * @author Xavier Blanc, Pierre Carpentier
 */
public abstract class PrintGenerator implements Generator {
	
	/** The output file name. */
	protected PrintWriter out;
	
	/** The output file name. */
	protected String filename = new String();

	/** The tabulation. */
	protected final String TABULATION = new String("    ");

	/** The identation level. */
	protected int level = 0;

	/** New Line indicator. */
	protected boolean newLine = true;
	

	/**
	 * Set the output file name.
	 * @param file The file name.
	 * @throws java.io.IOException if the file exists but is a directory rather than a regular file,
	 *                  does not exist but cannot be created, or cannot be opened for any other reason.
	 */
	public void setOutput(String file) throws IOException {
		out = new PrintWriter(new FileOutputStream(file), true);
		filename = file;
	}

	/**
	 * Set the output file.
	 * @param file The file.
	 * @throws java.io.IOException if the file exists but is a directory rather than a regular file,
	 *                  does not exist but cannot be created, or cannot be opened for any other reason.
	 */
	public void setOutput(java.io.File file) throws IOException {
		out = new PrintWriter(new FileOutputStream(file), true);
		filename = file.getName();
	}

	/**
	 * Set the output.
	 * @param stream The output stream.
	 */
	public void setOutput(OutputStream stream) throws IOException {
		out = new PrintWriter(stream);
	}

	/**
	 * Print the output to the global PrintWriter (replace out.print(String)).
	 * This default implementation generates no indentation.
	 * @param outputString The string to print.
	 */
	protected void output(String outputString) {
		int index = outputString.indexOf("\n");
		if (index != -1 && index != outputString.length() - 1) {
			// outputString contains at least '\n' which is not at the end
			java.util.StringTokenizer token = new java.util.StringTokenizer(outputString, "\n");
			while (token.hasMoreTokens()) {
				String next = token.nextToken();
				if (!token.hasMoreTokens()) {
					if (outputString.endsWith("\n"))
						outputln(next);
					else
						output(next);
				} else {
					outputln(next);
				}
			}
		} else {
			out.print(outputString);
			if (outputString.trim().endsWith("\n"))
				newLine = true;
			else
				newLine = false;
		}
	}

	/**
	 * Print a new line to the global PrintWriter (replace out.println()).
	 */
	protected void outputln() {
		out.println();
		newLine = true;
	}

	/**
	 * Print the output with a new line to the global PrintWriter (replace out.println(String)).
	 * @param outputString The string to output.
	 */
	protected void outputln(String outputString) {
		output(outputString);
		outputln();
	}
	
    /**
	 * Flush the output file.
	 */
    public void flushFile() {
	    out.flush();
    }
	
}
