/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.logging;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;

/**
 * This class represents the Logger used by the ModFact project.
 * It delegates the log process to the log framework, thus only
 * this class would be modified if the log framework changed.
 * 
 * @author Pierre Carpentier
 */
public class ModFactLogger {

	/** The delegates logger. */
	private Logger logger;

	/** The factory of logger. */
	private static LoggerFactory loggerFactory = getLoggerFactory();
	
	/** The quiet flag indicates if the logger is in quiet or in normal mode. */
	private static boolean isQuiet = false;
	
	/** The default file name of the properties file. */
	private static String configFile;

	/**
	 * Constructor of a named logger.
	 * @param name The name of the logger.
	 */
	public ModFactLogger(String name) {
		if (loggerFactory != null)
			logger = loggerFactory.getLogger(name);
	}

	/**
	 * Logs a message with the appropriate level.
	 * @param level The level of logging.
	 * @param message The message to log.
	 */
	public void log(int level, String message) {
		if (logger != null && !isQuiet) {
			switch (level) {
				case Level.FATAL :
					logger.log(BasicLevel.FATAL, message);
					break;
				case Level.ERROR :
					logger.log(BasicLevel.ERROR, message);
					break;
				case Level.WARN :
					logger.log(BasicLevel.WARN, message);
					break;
				case Level.INFO :
					logger.log(BasicLevel.INFO, message);
					break;
				case Level.FINE :
					logger.log(BasicLevel.DEBUG, message);
					break;
				case Level.FINER :
					logger.log(BasicLevel.INHERIT, message);
					break;
			}
		}
	}

	/**
	 * Gets a logger from its name (or creates it if the name is unknown).
	 * @param name The name of the logger.
	 * @return The logger.
	 */
	public static ModFactLogger getLogger(String name) {
		return new ModFactLogger(name);
	}
	
	/**
	 * Initializes the ModFactLogger.
	 */
	public static void init() {
		configFile = System.getProperty(Monolog.MONOLOG_FILE_NAME, Monolog.DEFAULT_MONOLOG_FILE);
		Monolog.init();
	}

	/**
	 * Gets the logger factory defined by the class name
	 * set in the property log.config.classname. <br>
	 * This variable can be set to :
	 * - org.objectweb.util.monolog.wrapper.javaLog.LoggerFactory
	 * for use the javaLog service. <br>
	 * - org.objectweb.util.monolog.wrapper.log4j.MonologLoggerFactory
	 * for use the log4j library.
	 * @return The factory of logger.
	 */
	public static LoggerFactory getLoggerFactory() {
		init();
		// load properties from config file
		Properties props = new Properties();
		String loggerFactoryName = null;
		try {
			props.load(new java.io.FileInputStream(configFile));
			loggerFactoryName = props.getProperty("log.config.classname", null);
		} catch (IOException ioe) {
			// Maybe Web Application
			try {
				String url = System.getProperty("org.objectweb.modfact.resourceURL")+"/"+configFile;
			URL webPath = new URL(url);
			props.load(webPath.openStream());
			loggerFactoryName = props.getProperty("log.config.classname", null);
			}
			catch (MalformedURLException ex) {
			}
			catch (IOException ex) {
			}
		}
		
		if (loggerFactoryName == null) {
			loggerFactoryName = System.getProperty("log.config.classname",
								"org.objectweb.util.monolog.wrapper.javaLog.LoggerFactory");
		}
		try {
			LoggerFactory lf = (LoggerFactory) Class.forName(loggerFactoryName).newInstance();
			return lf;
		} catch (Exception e) {
			System.err.println("The logger factory class name is not correct.");
			return null;
		}
	}
	
	/**
	 * Sets the quiet flag.
	 * TRUE to sets the logger in a quiet mode
	 * FALSE to sets the logger in a normal mode.
	 */
	public static void setQuiet (boolean quietMode) {
		isQuiet = quietMode;
	}

}
