/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.corba.provider;

import org.objectweb.modfact.corba.helper.IDLCommon;
import org.objectweb.modfact.corba.helper.JavaCommon;
import org.objectweb.modfact.corba.helper.MOFCommon;
import org.objectweb.modfact.corba.helper.XMIImportCommon;

/**
 * @author Pierre Carpentier
 *
 */
public class XMIImportProvider implements XMIImportCommon {

	/** The Helper. */
	private IDLCommon idlHelper;
	private MOFCommon mofHelper;
	private JavaCommon javaHelper;
	
	/**
	 * Default Constructor of the XMIImport Provider
	 */
	public XMIImportProvider () {
	}
	
	/**
	 * Constructor of the XMIImport Provider
	 * @param idlHelper The IDL helper.
	 * @param mofHelper The MOF helper.
	 * @param javaHelper The Java helper.
	 */
	public XMIImportProvider (IDLCommon idlHelper, MOFCommon mofHelper, JavaCommon javaHelper) {
		this.idlHelper = idlHelper;
		this.mofHelper = mofHelper;
		this.javaHelper = javaHelper;
	}

	/**
	 * Pass from a TypeCode to the corresponding Java type code
	 * @param _type The CORBA Type Code.
	 * @return A String representing the corresponding Java type.
	 */
	public String typeCode2Java(org.omg.CORBA.TypeCode _type) {
		switch (_type.kind().value()) {
			case org.omg.CORBA.TCKind._tk_boolean :
				return "boolean";
			case org.omg.CORBA.TCKind._tk_octet :
				return "byte";
			case org.omg.CORBA.TCKind._tk_char :
			case org.omg.CORBA.TCKind._tk_wchar :
				return "char";
			case org.omg.CORBA.TCKind._tk_short :
			case org.omg.CORBA.TCKind._tk_ushort :
				return "short";
			case org.omg.CORBA.TCKind._tk_long :
			case org.omg.CORBA.TCKind._tk_ulong :
				return "int";
			case org.omg.CORBA.TCKind._tk_longlong :
			case org.omg.CORBA.TCKind._tk_ulonglong :
				return "long";
			case org.omg.CORBA.TCKind._tk_double :
				return "double";
			case org.omg.CORBA.TCKind._tk_float :
				return "float";
			case org.omg.CORBA.TCKind._tk_string :
			case org.omg.CORBA.TCKind._tk_wstring :
				return "String";
			default :
				return "Object";
		}
	}

	/**
	 * Replace the first occurence of the given string with the given replacement.
	 * @param original The original String to modify.
	 * @param to_replace The substring of the string to search and replace.
	 * @param new_value The replacement String.
	 * @return The modified String.
	 */
	public String replaceFirst(String original, String to_replace, String new_value) {
		int index = original.indexOf(to_replace);
		if (index != -1) {
			return original.substring(0, index) + new_value + original.substring(index + to_replace.length());
		} else {
			return original;
		}
	}

	public String format1FirstMin(String toFormat) {
		String format1 = idlHelper.format1(toFormat);
		return format1.substring(0, 1).toLowerCase() + format1.substring(1);
	}

	/**
	 * Parse a String and test if it corresponds to the value. This value can be multiple and separate by '|', for example,
	 * matchesValues(to_parse, new String("value1|value2|value3")).
	 * @param to_parse The substring of the string to search and replace.
	 * @param value The replacement String.
	 * @return TRUE if the parsed string corresponds to one of the values.
	 */
	public boolean matchesValues(String to_parse, String value) {
		java.util.StringTokenizer token = new java.util.StringTokenizer(value, "|");
		while (token.hasMoreTokens()) {
			if (to_parse.equals(token.nextToken()))
				return true;
		}
		return false;
	}

	/**
	 * Parse a String and test if it ends with the value. This value can be multiple and separate by '|', for example,
	 * matchesValues(to_parse, new String("end1|end2|end3")).
	 * @param to_parse The substring of the string to search and replace.
	 * @param value The replacement String.
	 * @return TRUE if the parsed string ends with on of the values.
	 */
	public boolean endsWithValues(String to_parse, String value) {
		java.util.StringTokenizer token = new java.util.StringTokenizer(value, "|");
		while (token.hasMoreTokens()) {
			if (to_parse.endsWith(token.nextToken()))
				return true;
		}
		return false;
	}

	/** 
	 * Add a value to a hashtable composed by an object as key (a model) and a vector as value (models keys).
	 * @param hashtable The hashtable.
	 * @param key The key in the hastable.
	 * @param object The referenced object to add to the vector corresponding to the key of the hashtable.
	 */
	public void addHashtableValue(java.util.Hashtable hashtable, Object key, Object object) {
		if (hashtable.containsKey(key)) {
			java.util.Vector vector = (java.util.Vector) hashtable.get(key);
			if (!vector.contains(object))
				vector.add(object);
		} else {
			java.util.Vector vector = new java.util.Vector();
			vector.add(object);
			hashtable.put(key, vector);
		}
	}

	/**
	 * Tests if a MOF Object is contained in a vector.
	 * @param _vector The vector to parse.
	 * @param _object The MOF Object.
	 * @return TRUE if the vector contains the MOF Object.
	 */
	public boolean vectorContains(java.util.Vector _vector, org.omg.mof.Reflective.RefBaseObject _object) {
		if (_vector.size() == 0)
			return false;
		int i = 0;
		boolean _is_contained = false;
		while (!_is_contained && (i < _vector.size())) {
			org.omg.mof.Reflective.RefBaseObject _current = (org.omg.mof.Reflective.RefBaseObject) _vector.elementAt(i);
			if (_current.ref_itself(_object))
				_is_contained = true;
			else
				i++;
		}
		return _is_contained;
	}

	/**
	 * Tests if a MOF Object is contained in a vector.
	 * @param _array The array to parse.
	 * @param _object The MOF Object.
	 * @return TRUE if the vector contains the MOF Object.
	 */
	public boolean arrayContains(org.omg.CORBA.Object[] _array, org.omg.mof.Reflective.RefBaseObject _object) {
		for (int i = 0; i < _array.length; i++) {
			if (_array[i]._is_equivalent(_object))
				return true;
		}
		return false;
	}

	/**
	 * Gets the default value of an Attribute.
	 * @param _type The CORBA Type Code of the attribute.
	 * @return The default value of the attribute.
	 */
	public String defaultAttributeValue(org.omg.CORBA.TypeCode _type) throws org.omg.mof.Reflective.MofError {
		return defaultAttributeValue(_type, null);
	}

	/**
	 * Gets the default value of an Attribute.
	 * @param _type The CORBA Type Code of the attribute.
	 * @param _classifier The Object type of the attribute.
	 * @return The default value of the attribute.
	 */
	public String defaultAttributeValue(org.omg.CORBA.TypeCode _type, org.omg.mof.Model.Classifier _classifier)
		throws org.omg.mof.Reflective.MofError {
		try {
			switch (_type.kind().value()) {
				case org.omg.CORBA.TCKind._tk_boolean :
					return "false";
				case org.omg.CORBA.TCKind._tk_octet :
				case org.omg.CORBA.TCKind._tk_short :
				case org.omg.CORBA.TCKind._tk_ushort :
				case org.omg.CORBA.TCKind._tk_long :
				case org.omg.CORBA.TCKind._tk_ulong :
				case org.omg.CORBA.TCKind._tk_longlong :
				case org.omg.CORBA.TCKind._tk_ulonglong :
					return "0";
				case org.omg.CORBA.TCKind._tk_double :
				case org.omg.CORBA.TCKind._tk_float :
					return "0.0";
				case org.omg.CORBA.TCKind._tk_char :
				case org.omg.CORBA.TCKind._tk_wchar :
					return "''";
				case org.omg.CORBA.TCKind._tk_string :
				case org.omg.CORBA.TCKind._tk_wstring :
					return "\"\"";
				case org.omg.CORBA.TCKind._tk_alias :
					return defaultAttributeValue(_type.content_type(), _classifier);
				case org.omg.CORBA.TCKind._tk_enum :
					if (_type.member_count() > 0)
						return javaHelper.javaQualifiedName(_classifier) + "." + _type.member_name(0);
					else {
						return "null";
					}
				case org.omg.CORBA.TCKind._tk_TypeCode :
					return "_orb.create_any().type()";
				default :
					String classifierName = _classifier.name();
					if (isPrimitiveType(classifierName)) {
						if (classifierName.equalsIgnoreCase("String"))
							return "new String()";
						if (classifierName.equalsIgnoreCase("Boolean"))
							return "false";
						if (classifierName.equalsIgnoreCase("Integer") || classifierName.equalsIgnoreCase("Long"))
							return "0";
						if (classifierName.equalsIgnoreCase("Float") || classifierName.equalsIgnoreCase("Double"))
							return "0.0";
						return null;
					} else {
						return "null";
					}
			}
		} catch (Exception e) {
			return "null";
		}
	}

	/**
	 * Gets the default value of an Attribute.
	 * @param _classifier The Object type of the attribute.
	 * @return The default value of the attribute.
	 */
	public String defaultAttributeValue(org.omg.mof.Model.Classifier _classifier) throws org.omg.mof.Reflective.MofError {
		if (_classifier._is_a(org.omg.mof.Model.DataTypeHelper.id())) {
			org.omg.mof.Model.DataType dataType = org.omg.mof.Model.DataTypeHelper.narrow(_classifier);
			return defaultAttributeValue(dataType.type_code(), _classifier);
		} else {
			return "null";
		}
	}

	/**
	 * Tests if the DataType Name represents a PrimitiveType.
	 * @param datatype_name The name of the datatype.
	 */
	public boolean isPrimitiveType(String datatype_name) {
		if (datatype_name.startsWith("*"))
			return isPrimitiveType(datatype_name.substring(1));
		String tmp = datatype_name.toLowerCase();
		return (
			tmp.equalsIgnoreCase("string")
				|| tmp.equalsIgnoreCase("boolean")
				|| tmp.equalsIgnoreCase("integer")
				|| tmp.equalsIgnoreCase("long")
				|| tmp.equalsIgnoreCase("float")
				|| tmp.equalsIgnoreCase("double"));
	}

	/**
	 * Gets the references of an association.
	 * @param _association The association.
	 * @return The references.
	 */
	public org.omg.mof.Model.Reference[] getReferences(org.omg.mof.Model.Association _association)
		throws org.omg.mof.Reflective.MofError, org.omg.mof.Reflective.NotSet {
		java.util.Vector _temp = new java.util.Vector();
		// Get the package which contains the association
		org.omg.mof.Model.Package _package = org.omg.mof.Model.PackageHelper.narrow(_association.container());
		// Get all classes
		org.omg.mof.Model.Class _classes_of_package [];
		_classes_of_package = mofHelper.classesOfPackage(_package);
		// Get the AssociationEnd
		org.omg.mof.Model.AssociationEnd[] _association_ends = mofHelper.associationEndsOfAssociation(_association);
		for (int i = 0; i < _classes_of_package.length; i++) {
			org.omg.mof.Model.Class _class = (org.omg.mof.Model.Class) _classes_of_package[i];
			// already processed
			org.omg.mof.Model.Reference _references [];
			_references = mofHelper.referencesOfClass(_class);
			for (int j = 0; j < _references.length; j++) {
				org.omg.mof.Model.Reference _reference = (org.omg.mof.Model.Reference) _references[j];
				org.omg.mof.Model.AssociationEnd _association_end = _reference.referenced_end();
				if (_association_end._is_equivalent(_association_ends[0]) || _association_end._is_equivalent(_association_ends[1])) {
					_temp.addElement(_reference);
				}
			}
		}
		org.omg.mof.Model.Reference[] _referencees = new org.omg.mof.Model.Reference[_temp.size()];
		for (int i = 0; i < _referencees.length; i++)
			_referencees[i] = (org.omg.mof.Model.Reference) _temp.elementAt(i);
		return _referencees;
	}

}
