package org.objectweb.modfact.jmi.ps;
import javax.jmi.reflect.*;
import java.util.*;

import org.objectweb.modfact.jmi.logging.Level;
import org.objectweb.modfact.jmi.logging.ModFactLogger;
import org.objectweb.modfact.jmi.reflect.*;

/**
 * @author gregory
 *
 * This class is a tool for traverse any MOF Model based on the design pattern
 * Iterator. It builds a linear representation of the MOF model i.e put it in a
 * list. We do a BFS on the containment relationship with an order on types
 * (Package, Class, Association, and Objects).
 * 
 * 
 * There's a filter system, i.e you can only browse a certain type of the model.
 * This latter feature is improvable
 */
public class MOFPolymorphicIterator implements Iterator {

	private Vector allNodes = new Vector();
	private int index = 0;

	private boolean packageFilter = false;
	private boolean classFilter = false;
	private boolean associationFilter = false;
	private boolean instanceFilter = false;
	private ModFactLogger logger;

	/**
	 * 
	 * @param packageFilter
	 * @param classFilter
	 * @param associationFilter
	 * @param instanceFilter
	 * if one of the parameters is set to true then all elements of
	 * this type will be filtered in the traversal algorithm
	 */
	public void setFilters(
		boolean packageFilter,
		boolean classFilter,
		boolean associationFilter,
		boolean instanceFilter) {
		this.packageFilter = packageFilter;
		this.classFilter = classFilter;
		this.associationFilter = associationFilter;
		this.instanceFilter = instanceFilter;

	}

	/**
	 * 
	 * @param o : RefBaseObject
	 * Traverse all the contained objects of o, if o is not an instance
	 */
	public MOFPolymorphicIterator(RefBaseObject o) {

		Iterator it;

		allNodes.addElement(o);
		BFS(o);

		//debugCode
		this.logger = ModFactLogger.getLogger("fr.lip6.modfact.jmi.reflect.ps");
		logger.log(Level.INFO, "cration de l'itrateur");

		it = allNodes.iterator();

		while (it.hasNext()) {

			RefBaseObject oo = (RefBaseObject) it.next();

			if (!(oo instanceof RefObject))
				logger.log(
					Level.INFO,
					(oo.getClass().toString()
						+ "type de l'objet:"
						+ (((RefBaseObject) oo).refMetaObject()).refGetValue(
							"name"))
						+ " ");
			else if (oo instanceof RefObject)
				logger.log(
					Level.INFO,
					(oo.getClass().toString()
						+ "nom et type de l'objet:"
						+ ((RefObject) oo).refGetValue("name"))
						+ "----"
						+ ((RefObject) oo).refMetaObject().refGetValue("name"));
			else
				logger.log(Level.INFO, oo.getClass().toString() + ":" + "#");
		}

		index = 0;
	}

	/**
	 * 
	 * @see java.util.Iterator#hasNext()
	 * Implements the design pattern hasNext operation
	 */
	public boolean hasNext() {

		while (index < allNodes.size()) {
			if ((allNodes.elementAt(index) instanceof RefPackage
				&& packageFilter)
				|| ((allNodes.elementAt(index) instanceof RefClass
					&& classFilter))
				|| ((allNodes.elementAt(index) instanceof RefAssociation
					&& associationFilter))
				|| ((allNodes.elementAt(index) instanceof RefObject
					&& instanceFilter)))
				index++;
			else
				return true;

		}
		return false;
	}

	/**
	 * @see java.util.Iterator#next()
	 * implement the design pattern Iterator next operation
	 */

	public Object next() {

		RefBaseObject o = null;

		while (index < allNodes.size()) {
			if ((allNodes.elementAt(index) instanceof RefPackage
				&& packageFilter)
				|| ((allNodes.elementAt(index) instanceof RefClass
					&& classFilter))
				|| ((allNodes.elementAt(index) instanceof RefAssociation
					&& associationFilter))
				|| ((allNodes.elementAt(index) instanceof RefObject
					&& instanceFilter)))
				index++;
			else {

				return allNodes.elementAt(index++);

			}

		}
		return o;
	}

	/**
	 * Return the first element and reset the index 
	 * of the list
	 */
	public Object first() {
		index = 0;
		return (RefBaseObject) allNodes.elementAt(index);
	}

	/**
	 * We can reuse the iterator an browse again the model
	 * by resseting the index of the list
	 */
	public void reinitialize() {
		index = 0;
	}

	/**
	* @see java.util.Iterator#remove()
	*/
	public void remove() {
	}

	/**
	 * Method BFS.
	 * @param current
	 *
	 * Put all elements of the Model in a list by traversing the graph of
	 * containment relationship. We do a breadth first search.
	 */
	private void BFS(RefBaseObject current) {

		Iterator it;

		index++;
		if (hasPackages(current)) {
			it = ((RefPackage) current).refAllPackages().iterator();
			while (it.hasNext())
				allNodes.addElement(it.next());

		}
		if (hasClasses(current)) {
			it = ((RefPackage) current).refAllClasses().iterator();
			while (it.hasNext())
				allNodes.addElement(it.next());

		}
		if (hasAssociation(current)) {
			it = ((RefPackage) current).refAllAssociations().iterator();
			while (it.hasNext())
				allNodes.addElement(it.next());

		}

		if (hasInstance(current)) {
			it = ((RefClass) current).refAllOfClass().iterator();
			while (it.hasNext()) {
				allNodes.addElement(it.next());
			}

		}

		if (index < allNodes.size())
			BFS((RefBaseObject) allNodes.elementAt(index));

	}

	/**
	 * Method hasChild.
	 * @param p
	 * @return boolean
	 *
	 * The followings helper function should be improved
	 */
	private boolean hasChild(RefPackageImpl p) {

		if (!p.refAllPackages().isEmpty())
			return true;

		return false;

	}

	/**
	 * Method hasPackages.
	 * @param o
	 * @return boolean
	 */
	private boolean hasPackages(RefBaseObject o) {
		if ((o instanceof RefPackageImpl)
			&& !(((RefPackage) o).refAllPackages().isEmpty()))
			return true;

		return false;
	}

	/**
	 * Method hasClasses.
	 * @param o
	 * @return boolean
	 */
	private boolean hasClasses(RefBaseObject o) {
		if ((o instanceof RefPackageImpl)
			&& !(((RefPackage) o).refAllClasses().isEmpty()))
			return true;
		return false;
	}
	
	/**
	 * Method hasAssociation.
	 * @param o
	 * @return boolean
	 */
	private boolean hasAssociation(RefBaseObject o) {
		if ((o instanceof RefPackage)
			&& !(((RefPackage) o).refAllAssociations().isEmpty()))
			return true;
		return false;
	}

	/**
	 * Method hasInstance.
	 * @param o
	 * @return boolean
	 */
	private boolean hasInstance(RefBaseObject o) {
		if ((o instanceof RefClass)
			&& !(((RefClass) o).refAllOfClass().isEmpty()))
			return true;
		return false;
	}

}
