/**
 * copyright 2002 2004 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with ModFact; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.modfact.jmi.xmiio.exporter;

import javax.jmi.model.ModelElement;
import javax.jmi.reflect.RefObject;

import org.objectweb.modfact.jmi.generator.TagGenerator;
import org.objectweb.modfact.jmi.logging.Level;
import org.objectweb.modfact.jmi.logging.ModFactLogger;

/**
 * An abstract implementation of an XMI Import.
 * This class provides common functions for all XMI importers,
 * and generic functions for parsing files thanks to the XMIImportUtils inheritance
 */
public abstract class XMIExport extends TagGenerator {

	/** The DTD URI. */
	protected String _dtd;

	/** The model name. */
	protected String _model;

	/** The Meta-Model name. */
	protected String _metamodel;

	/** The different element exported into XMI with their keys (ref_mof_id / xmi_id). */
	protected java.util.Hashtable _elements = new java.util.Hashtable();

	/** The index for the xmi_id of the elements. */
	protected int _element_index = 1;

	/** The different TypeCode exported into XMI (TypeCode / xmi_id). */
	protected java.util.Hashtable _type_codes = new java.util.Hashtable();

	/** The index for the xmi_id of the TypeCodes. */
	protected int _type_code_index = 1;

	/** The Logger. */
	protected ModFactLogger _logger;

	/**
	 * Default XMI Export Constructor.
	 * @param dtd The DTD File.
	 * @param model The model name.
	 * @param metamodel The Meta-Model name.
	 */
	protected XMIExport(String dtd, String model, String metamodel) {
		super();
		_dtd = dtd;
		_model = model;
		_metamodel = metamodel;
	}
	/**
	 * Parse the package. The package is retrieve from its name in the Name Service.
	 * @param name The name in the Name Service.
	 */
	public void parseName(String name) {
        System.err.println("parseName(" + name + ")");
	}

	/**
	 * Parse the package.
	 * @param root_package The Root Package.
	 */
	public void parse(Object root_package) {
		outputln("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		if (_dtd.length() > 0) {
			try {
				java.net.URL url = new java.net.URL(_dtd);
				outputln("<!DOCTYPE XMI SYSTEM \"" + url.toString() + "\">");
			} catch (java.net.MalformedURLException murle) {
				murle.printStackTrace();
			}
		}
		outputln("<XMI verifier=\"false\" xmi.version=\"1.1\">");
		xmiHeaderTemplate();
		xmiContentTemplate(root_package);
		outputln("</XMI>");
		flushFile();
	}

	/**
	 * Generate the header of the XMI File.
	 */
	public void xmiHeaderTemplate() {
		_logger.log(Level.FINE, "XMI Header generation ...");
		outputln("<XMI.header>");
		outputln("<XMI.model xmi.name=\"" + _model + "\"/>");
		outputln("<XMI.metamodel xmi.name=\"" + _metamodel + "\" xmi.version=\"1.3\"/>");
		outputln("<XMI.documentation>");
		outputln("<XMI.exporter>Generated by the ModFact XMI exporter</XMI.exporter>");
		outputln("</XMI.documentation>");
		outputln("</XMI.header>");
	}

	/**
	 * Generate the content of the XMI File.
	 * @param _root_package The Root Package.
	 */
	public void xmiContentTemplate(Object _root_package) {
		_logger.log(Level.FINE, "XMI Content generation ...");
		outputln("<XMI.content>");
		rootPackageTemplate(_root_package);
		outputln("</XMI.content>");
	}

	/**
	 * The root package processing.
	 * @param _root_package The Root Package.
	 */
	public abstract void rootPackageTemplate(Object _root_package);

	// ==================================================================
	//  Common Functions
	// ==================================================================

	/**
	 * Get the XMI id for an object which is exported to an XMI file.
	 * @param object The object to export.
	 */
	protected String getXMIId(Object object) {
        RefObject refObject = (RefObject)object;
		String mof_id = refObject.refMofId();
		if (_elements.containsKey(mof_id))
			return (String) _elements.get(mof_id);
		String key = "a" + _element_index;
		_element_index++;
		_elements.put(mof_id, key);
		return key;
	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#generate()
	 */
	public void generate() throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setInput(org.omg.mof.Model.ModelElement[])
	 */
	public void setInput(ModelElement[] elt) throws Exception {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.objectweb.modfact.corba.generator.Generator#setLogger(org.objectweb.modfact.corba.logging.ModFactLogger)
	 */
	public void setLogger(ModFactLogger log) throws Exception {
		_logger = log;

	}

}
